use std::sync::Arc;
use rouille::router;
use crate::repo::pokemon::Repository;

mod health;
mod create_pokemon;


pub fn serve(url: &str, repo: Arc<dyn Repository>) {
    rouille::start_server(url, move |req| {
        router!(req,
            (POST)(/health) => {
                health::serve()
        },
            (POST)(/) => {
                create_pokemon::serve(repo.clone(), req)
        },
            _ => {
                rouille::Response::from(rouille::Response::empty_404())
            }
        )
    });
}

enum Status {
    BadRequest,
    NotFound,
    Conflict,
    InternalServerError,
}

impl From<Status> for rouille::Response {
    fn from(value: Status) -> Self {
        let status_code = match value {
            Status::BadRequest => 400,
            Status::NotFound => 404,
            Status::Conflict => 409,
            Status::InternalServerError => 500,
        };
        Self {
            status_code,
            headers: vec![],
            data: rouille::ResponseBody::empty(),
            upgrade: None,
        }
    }
}